Attribute VB_Name = "mDXGraphics"
Option Explicit

'Author
    Public Const NL As String = vbNewLine
    Public Const ModuleName As String = "DXGraphics module"
    
    Public Const Author As String = "Fox McCloud"
    Public Const Website As String = "http://fox.acky.net/"
    
    Public Const LicenseType As String = "Free"
    Public Const LicenseText As String = "You are allowed to modify and use this code in any way!"
    Public Const LicenseNote As String = "If this code was helpfull for you please include me in the credits."

'Constants
    Private Const Pi As Single = 3.14159
    Private Const Rad As Single = Pi / 180
    
    Private Const Sqr2 As Single = 1.41421
    
    Private Const cVertexFVF As Long = (D3DFVF_XYZRHW Or D3DFVF_DIFFUSE Or D3DFVF_TEX1)

'Types
    Private Type tVertex
        'Position
        x As Single
        y As Single
        z  As Single
        
        'Vertex data
        RHW As Single
        Diffuse As Long
        
        'Texture coordinates
        u As Single
        v As Single
    End Type
    
    Private Type tTexture
        'Internal
        FileName As String
        
        'Texture information
        Surface As Direct3DTexture8
        
        W As Single
        H As Single
    End Type
    
    Private Type tBitmapHeader
        ID As String * 2
        FileSize As Long
        Reserved As Long
        Offset As Long
        HeaderSize As Long
        
        'Size
        BmpWidth As Long
        BmpHeight As Long
    End Type
    
    Private Type tBuffer
        'Buffer setup
        Texture As Long
        
        'Contents
        VertexCount As Long
        Vertex() As tVertex
    End Type

'Variables
    'DirectX objects
    Private DirectX As DirectX8
    Private Direct3D As Direct3D8
    Private D3DX As New D3DX8
    
    Private VertexSize As Long
    
    Private ActTexture As Long
    Private TextureCount As Long
    Private Texture() As tTexture
    
    Private Buffer() As tBuffer
    Private BufferCount As Long

    'Public
    Public Device As Direct3DDevice8
    
    Public Ambient As D3DCOLORVALUE
    
Public Sub BeginScene()
    'Begin scene
    Device.Clear 0, ByVal 0, D3DCLEAR_TARGET, &H0, 1#, 0
    Device.BeginScene
End Sub

Public Sub Blt(iTexture As Long, iX As Single, iY As Single, iW As Single, iH As Single, iSourceX As Single, iSourceY As Single, iSourceW As Single, iSourceH As Single, Optional iDiffuseColor As Long = &HFFFFFF, Optional iRotation As Single = 0)
    Dim A As Long
    
    Dim Vertex(5) As tVertex
    Dim VertexBuffer As Direct3DVertexBuffer8
    
    If iW <= 0 Then: iW = Texture(iTexture).W
    If iH <= 0 Then: iH = Texture(iTexture).H
    
    Dim OffsetU As Single
    Dim OffsetV As Single
    
    Dim ScaleW As Single
    Dim ScaleH As Single
    
    'Calculate UV factor
    ScaleW = iSourceW / Texture(iTexture).W
    ScaleH = iSourceH / Texture(iTexture).H
    
    OffsetU = iSourceX / Texture(iTexture).W
    OffsetV = iSourceY / Texture(iTexture).H
    
    If iRotation = 0 Then
        'Calculate edge points
        With Vertex(0): .x = iX: .y = iY: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV: .Diffuse = iDiffuseColor: End With
        With Vertex(1): .x = iX + iW: .y = iY: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV: .Diffuse = iDiffuseColor: End With
        With Vertex(2): .x = iX + iW: .y = iY + iH: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
        With Vertex(5): .x = iX: .y = iY + iH: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
        
    Else
        Dim OffsetX As Single
        Dim OffsetY As Single
        
        Dim ZoomX As Single
        Dim ZoomY As Single
        
        'Pre-calculate offset
        OffsetX = iX + (iW / 2)
        OffsetY = iY + (iH / 2)
        
        'This is used to keep the picture size
        ZoomX = (iW * Sqr2 / 2)
        ZoomY = (iH * Sqr2 / 2)
        
        'Calculate edge points
        With Vertex(0): .x = OffsetX + Cos(iRotation - 135 * Rad) * ZoomX: .y = OffsetY + Sin(iRotation - 135 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV: .Diffuse = iDiffuseColor: End With
        With Vertex(1): .x = OffsetX + Cos(iRotation + 135 * Rad) * ZoomX: .y = OffsetY + Sin(iRotation + 135 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV: .Diffuse = iDiffuseColor: End With
        With Vertex(2): .x = OffsetX + Cos(iRotation + 45 * Rad) * ZoomX: .y = OffsetY + Sin(iRotation + 45 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
        With Vertex(5): .x = OffsetX + Cos(iRotation - 45 * Rad) * ZoomX: .y = OffsetY + Sin(iRotation - 45 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
    End If
    
    'Copy other points
    Vertex(3) = Vertex(0)
    Vertex(4) = Vertex(2)
    
    'Create vertex buffer
    Set VertexBuffer = Device.CreateVertexBuffer(VertexSize * 6, 0, cVertexFVF, D3DPOOL_DEFAULT)
    If VertexBuffer Is Nothing Then: Exit Sub
    
    'Setup buffer
    D3DVertexBuffer8SetData VertexBuffer, 0, VertexSize * 6, 0, Vertex(0)
    
    With Device
        If Not ActTexture = iTexture Then
            'Set texture
            ActTexture = iTexture
            .SetTexture 0, Texture(ActTexture).Surface
        End If
        
        'Render mesh
        .SetStreamSource 0, VertexBuffer, VertexSize
        .DrawPrimitive D3DPT_TRIANGLELIST, 0, 2
    End With
    
    'Release memory
    Set VertexBuffer = Nothing
End Sub

Public Sub DrawText(iFontBitmap As Long, iText As String, iX As Long, iY As Long, iSize As Long, iColor As Long, iRotation As Single)
    Dim A As Long
    Dim Pos As Long
    Dim ABC As String
    
    Dim OffX As Long
    Dim OffY As Long
    
    Dim CharsX As Long
    Dim CharSize As Single
    
    'Font setup
    ABC = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789,.!?:/\""'+-=@*&^$#_()|<>~ "
    CharsX = 8
    CharSize = 32
    
    'Position offset
    OffX = iX - (Len(iText) * iSize / 2)
    OffY = iY - (iSize / 2)
    
    For A = 1 To Len(iText)
        Pos = InStr(1, ABC, Mid(iText, A, 1), vbTextCompare) - 1
        
        If Pos >= 0 Then
            BltToBuffer iFontBitmap, CSng(OffX + (A * iSize)), CSng(OffY), CSng(iSize), CSng(iSize), CSng(CLng(Pos Mod CharsX) * CharSize), CSng(Int(Pos / CharsX) * CharSize), CharSize, CharSize, iColor, iRotation
        End If
    Next
    
    BltBuffers
End Sub

Public Sub EndScene()
    'Flip
    Device.EndScene
    Device.Present ByVal 0, ByVal 0, 0, ByVal 0
End Sub

Public Function InitWindowed(hWnd As Long) As Boolean
    'Set display mode
    Dim DisplayMode As D3DDISPLAYMODE
    Direct3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, DisplayMode
    
    'Set present parameters
    Dim PresentParameters As D3DPRESENT_PARAMETERS
    With PresentParameters
        .Windowed = 1 'Windowed mode
        .SwapEffect = D3DSWAPEFFECT_COPY_VSYNC
        .BackBufferFormat = DisplayMode.Format
    
        'Enable ZBuffer
        .EnableAutoDepthStencil = 1
        If Direct3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DisplayMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D16) = D3D_OK Then
            'Uset 16-bit ZBuffer
            .AutoDepthStencilFormat = D3DFMT_D16
            
        Else
            'Find other AutoDepth modes here
            Exit Function
        End If
    End With
    
    'Create device
    Set Device = Direct3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, PresentParameters)
    If Device Is Nothing Then: Exit Function
    
    With Device
        .SetVertexShader cVertexFVF
        
        'Setup device
        .SetRenderState D3DRS_CULLMODE, D3DCULL_CCW
        .SetRenderState D3DRS_LIGHTING, 1
        .SetRenderState D3DRS_ALPHABLENDENABLE, 1
        .SetRenderState D3DRS_COLORVERTEX, 0
        .SetRenderState D3DRS_SPECULARENABLE, 0
        
        .SetRenderState D3DRS_ALPHAREF, &H8&
        .SetRenderState D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL
    End With
    
    'Success
    InitWindowed = True
End Function

Public Function LoadTexture(iFilename As String) As Long
On Error Resume Next

    Dim A As Long
    Dim BitmapHeader As tBitmapHeader
    
    For A = 0 To TextureCount
        'Find loaded texture
        If StrComp(Texture(A).FileName, iFilename, vbTextCompare) = 0 Then
            LoadTexture = A
            
            Exit Function
        End If
    Next

    'Allocate memory
    TextureCount = TextureCount + 1
    ReDim Preserve Texture(TextureCount)
    
    With Texture(TextureCount)
        'Load textures
        Set .Surface = D3DX.CreateTextureFromFileEx(Device, iFilename, 0, 0, 1, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, D3DColorRGBA(255, 255, 255, 0), ByVal 0, ByVal 0)
        
        'Get bitmap size
        Open iFilename For Binary As 1: Get 1, , BitmapHeader: Close 1
        
        'Apply values
        .FileName = LCase(iFilename)
        
        'Size
        .W = BitmapHeader.BmpWidth
        .H = BitmapHeader.BmpHeight
    End With
    
    'Return index
    LoadTexture = TextureCount
End Function

Public Function Initialize() As Boolean
On Error Resume Next

    'Initialize variables
    TextureCount = -1
    BufferCount = -1
    
    'Get vertex size
    Dim TempVertex As tVertex
    VertexSize = Len(TempVertex)
    
    'Ambient light
    With Ambient: .r = 255: .g = 255: .B = 255: .A = 255: End With
    
    'Create DirectX
    If DirectX Is Nothing Then: Set DirectX = New DirectX8
    If DirectX Is Nothing Then: Exit Function
    
    'Create Direct3D
    If Direct3D Is Nothing Then: Set Direct3D = DirectX.Direct3DCreate
    If Direct3D Is Nothing Then: Exit Function
    
    'Create Direct3D helper
    If D3DX Is Nothing Then: Set D3DX = New D3DX8
    If D3DX Is Nothing Then: Exit Function
    
    'Success
    Initialize = True
End Function

Public Sub BltToBuffer(iTexture As Long, iX As Single, iY As Single, iW As Single, iH As Single, iSourceX As Single, iSourceY As Single, iSourceW As Single, iSourceH As Single, Optional iDiffuseColor As Long = &HFFFFFF, Optional iRotation As Single = 0)
    Dim A As Long
    Dim Index As Long
    
    If iW <= 0 Then: iW = Texture(iTexture).W
    If iH <= 0 Then: iH = Texture(iTexture).H
    
    Dim OffsetU As Single
    Dim OffsetV As Single
    
    Dim ScaleW As Single
    Dim ScaleH As Single
    
    'Calculate UV factor
    ScaleW = iSourceW / Texture(iTexture).W
    ScaleH = iSourceH / Texture(iTexture).H
    
    OffsetU = iSourceX / Texture(iTexture).W
    OffsetV = iSourceY / Texture(iTexture).H
    
    For A = 0 To BufferCount
        'Find existing buffer
        If Buffer(A).Texture = iTexture Then: Exit For
    Next
    
    If A > BufferCount Then
        'Create new buffer
        BufferCount = BufferCount + 1
        ReDim Preserve Buffer(BufferCount)
        
        'Setup buffer
        Buffer(BufferCount).Texture = iTexture
        Buffer(BufferCount).VertexCount = -1
    End If
    
    With Buffer(A)
        'Allocate memory
        Index = (.VertexCount + 1)
        .VertexCount = .VertexCount + 6
        ReDim Preserve .Vertex(.VertexCount)
        
        If iRotation = 0 Then
            'Calculate edge points
            With .Vertex(Index): .x = iX: .y = iY: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV: .Diffuse = iDiffuseColor: End With
            With .Vertex(Index + 1): .x = iX + iW: .y = iY: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV: .Diffuse = iDiffuseColor: End With
            With .Vertex(Index + 2): .x = iX + iW: .y = iY + iH: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
            With .Vertex(Index + 5): .x = iX: .y = iY + iH: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
            
        Else
            Dim OffsetX As Single
            Dim OffsetY As Single
            
            Dim ZoomX As Single
            Dim ZoomY As Single
            
            'Pre-calculate offset
            OffsetX = iX + (iW / 2)
            OffsetY = iY + (iH / 2)
            
            'Zoom to keep the picture size when rotating
            ZoomX = (iW * Sqr2 / 2)
            ZoomY = (iH * Sqr2 / 2)
            
            'Calculate edge points
            With .Vertex(Index): .x = OffsetX + Sin(iRotation - 135 * Rad) * ZoomX: .y = OffsetY + Cos(iRotation - 135 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV: .Diffuse = iDiffuseColor: End With
            With .Vertex(Index + 1): .x = OffsetX + Sin(iRotation + 135 * Rad) * ZoomX: .y = OffsetY + Cos(iRotation + 135 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV: .Diffuse = iDiffuseColor: End With
            With .Vertex(Index + 2): .x = OffsetX + Sin(iRotation + 45 * Rad) * ZoomX: .y = OffsetY + Cos(iRotation + 45 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU + ScaleW: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
            With .Vertex(Index + 5): .x = OffsetX + Sin(iRotation - 45 * Rad) * ZoomX: .y = OffsetY + Cos(iRotation - 45 * Rad) * ZoomY: .z = 0: .RHW = 1: .u = OffsetU: .v = OffsetV + ScaleH: .Diffuse = iDiffuseColor: End With
        End If
        
        'Copy other points
        .Vertex(Index + 3) = .Vertex(Index)
        .Vertex(Index + 4) = .Vertex(Index + 2)
    End With
End Sub

Public Sub BltBuffers()
    Dim A As Long
    
    Dim VertexBuffer As Direct3DVertexBuffer8
    
    For A = 0 To BufferCount
        With Buffer(A)
            'Create vertex buffer
            Set VertexBuffer = Device.CreateVertexBuffer(VertexSize * (.VertexCount + 1), 0, cVertexFVF, D3DPOOL_DEFAULT)
            If VertexBuffer Is Nothing Then: Exit Sub
            
            'Setup buffer
            D3DVertexBuffer8SetData VertexBuffer, 0, VertexSize * (.VertexCount + 1), 0, .Vertex(0)
            
            With Device
                'Set texture
                ActTexture = Buffer(A).Texture
                .SetTexture 0, Texture(ActTexture).Surface
                
                'Render buffer
                .SetStreamSource 0, VertexBuffer, VertexSize
                .DrawPrimitive D3DPT_TRIANGLELIST, 0, (Buffer(A).VertexCount + 1) / 3
            End With
            
            'Release memory
            Set VertexBuffer = Nothing
        End With
    Next
    
    'Clear all buffers
    BufferCount = -1
    Erase Buffer
End Sub

Public Sub ShutDown()
    Dim A As Long
    
    'Release memory
    For A = 0 To TextureCount: Set Texture(A).Surface = Nothing: Next
    TextureCount = -1
    
    'Clear all buffers
    BufferCount = -1
    Erase Buffer
    
    'Release objects
    Set Device = Nothing
    
    'Release DirectX
    Set D3DX = Nothing
    Set Direct3D = Nothing
    Set DirectX = Nothing
End Sub

