Attribute VB_Name = "mSound"
Option Explicit

'Const
    Global Const WAVEMIX_MAXCHANNELS = 8

'Types
    Type tChannelInfo
        Loops As Long
        WaveFile As String
    End Type
    
    
    Type WAVEMIXINFO
        wSize As Integer
        dwFormats As Long
        szDate(12) As String
        bVersionMajor As String * 1
        bVersionMinor As String * 1
    End Type
    
    Type MIXCONFIG
        wSize As Integer
        dwFlagsLo As Integer
        dwFlagsHi As Integer
        wChannels As Integer
        wSamplingRate As Integer
    End Type
    
    Private Type MIXPLAYPARAMS
        wSize As Integer
        hMixSessionLo As Integer
        hMixSessionHi As Integer
        iChannelLo As Integer
        iChannelHi As Integer
        lpMixWaveLo As Integer
        lpMixWaveHi As Integer
        hWndNotifyLo As Integer
        hWndNotifyHi As Integer
        dwFlagsLo As Integer
        dwFlagsHi As Integer
        wLoops As Integer
    End Type

'API
    Declare Sub WaveMixPump Lib "WAVMIX32.DLL" ()
    Declare Function WaveMixInit Lib "WAVMIX32.DLL" () As Long
    Declare Function WaveMixConfigureInit Lib "WAVMIX32.DLL" (lpConfig As MIXCONFIG) As Long
    Declare Function WaveMixActivate Lib "WAVMIX32.DLL" (ByVal hMixSession As Long, ByVal fActivate As Integer) As Long
    Declare Function WaveMixOpenWave Lib "WAVMIX32.DLL" (ByVal hMixSession As Long, szWaveFilename As Any, ByVal hInst As Long, ByVal dwFlags As Long) As Long
    Declare Function WaveMixOpenChannel Lib "WAVMIX32.DLL" (ByVal hMixSession As Long, ByVal iChannel As Long, ByVal dwFlags As Long) As Long
    Declare Function WaveMixPlay Lib "WAVMIX32.DLL" (lpMixPlayParams As Any) As Integer
    Declare Function WaveMixFlushChannel Lib "WAVMIX32.DLL" (ByVal hMixSession As Long, ByVal iChannel As Integer, ByVal dwFlags As Long) As Integer
    Declare Function WaveMixCloseChannel Lib "WAVMIX32.DLL" (ByVal hMixSession As Long, ByVal iChannel As Integer, ByVal dwFlags As Long) As Integer
    Declare Function WaveMixFreeWave Lib "WAVMIX32.DLL" (ByVal hMixSession As Long, ByVal lpMixWave As Long) As Integer
    Declare Function WaveMixCloseSession Lib "WAVMIX32.DLL" (ByVal hMixSession As Long) As Integer
    Declare Function WaveMixGetInfo Lib "WAVMIX32.DLL" (lpWaveMixInfo As WAVEMIXINFO) As Integer

'Vars
    Dim ChannelCount As Long
    
    Dim hWaveMix As Long
    Dim lpWaveMix() As Long
    Dim WaveHandle As Long
    Dim WAVMIX_Quiet As Integer
Private Function HiWord(ByVal l As Long) As Integer
    l = l \ &H10000
    
    HiWord = Val("&H" & Hex$(l))
End Function

Private Function LoWord(ByVal l As Long) As Integer
    l = l And &HFFFF&
    
    LoWord = Val("&H" & Hex$(l))
End Function


Function WAVMIX_AddFile(FileName As String) As Integer
    'Open a wave file and assign it to the next available
    'channel.

    Dim Temp As Long

    WAVMIX_AddFile = False
    If WAVMIX_Quiet Then: Exit Function
    If WaveHandle + 1 = WAVEMIX_MAXCHANNELS Then: Exit Function

    ReDim Preserve lpWaveMix(WaveHandle)
    lpWaveMix(WaveHandle) = WaveMixOpenWave(hWaveMix, ByVal FileName, 0, 0)
    Temp = WaveMixOpenChannel(hWaveMix, WaveHandle, 0)
    WAVMIX_AddFile = WaveHandle
    WaveHandle = WaveHandle + 1
End Function

Public Sub WAVMIX_SetFile(FileName As String, AChannel As Long)
    'Assign a new wave file, FileName, to the specified channel,
    'AChannel.  If this channel is currently assigned another
    'wave file, stop playing the channel and free the active
    'wave file.

    Dim Temp As Long
    
    If WAVMIX_Quiet Then: Exit Sub
    
    If AChannel > UBound(lpWaveMix) Then
        ReDim Preserve lpWaveMix(AChannel)
        WaveHandle = AChannel
    End If
    
    'If another wave is currently assigned to this channel, free it.
    If lpWaveMix(AChannel) <> 0 Then
        WAVMIX_StopChannel AChannel
        Temp = WaveMixFreeWave(hWaveMix, lpWaveMix(AChannel))
    End If
    
    ' Open the new wave and assign it to this channel.
    lpWaveMix(AChannel) = WaveMixOpenWave(hWaveMix, ByVal FileName, 0, 0)
    Temp = WaveMixOpenChannel(hWaveMix, AChannel, 0)
End Sub


Public Sub WAVMIX_Close()
    ' Stop playing all channels and free all wave files, then
    ' close down this WaveMix session.
    
    Dim Temp As Long
    Dim A As Long
    Dim rc As Integer
    
    If WAVMIX_Quiet Then: Exit Sub

    If hWaveMix <> 0 Then
        For A = 0 To UBound(lpWaveMix)
            If lpWaveMix(A) <> 0 Then
                WAVMIX_StopChannel A
                rc = WaveMixFreeWave(hWaveMix, lpWaveMix(A))
            End If
        Next
        
        Temp = WaveMixCloseSession(hWaveMix)
        hWaveMix = 0
    End If
End Sub

Function WAVMIX_InitMixer() As Integer
    'Initialize and activate the WaveMix DLL.
    
    Dim Temp As Long
    Dim Config As MIXCONFIG

    If WAVMIX_Quiet Then: Exit Function

    WaveHandle = 0
    ReDim lpWaveMix(0)
    
    Config.wSize = Len(Config)
    Config.dwFlagsHi = 1
    Config.dwFlagsLo = 0
    
    'Allow stereo sound
    Config.wChannels = 2
    hWaveMix = WaveMixConfigureInit(Config)
    Temp = WaveMixActivate(hWaveMix, True)

    If Temp <> 0 Then
        WAVMIX_InitMixer = False
        WaveMixCloseSession hWaveMix
        hWaveMix = 0
        
    Else
        WAVMIX_InitMixer = True
    End If
    
    ChannelCount = -1
End Function

Public Sub WAVMIX_StopChannel(ByVal ChannelNum As Long)
    'Stop playing the specified channel.
    
    Dim Temp As Integer

    If WAVMIX_Quiet Then: Exit Sub
    If hWaveMix = 0 Then: Exit Sub
    
    Temp = WaveMixFlushChannel(hWaveMix, ChannelNum, 0)
End Sub

Public Sub WAVMIX_Activate(Activate As Long)
    'Activate WavMix
    
    Dim Temp As Integer

    If WAVMIX_Quiet Then: Exit Sub
    If (hWaveMix = 0) Then: Exit Sub

    Temp = WaveMixActivate(hWaveMix, Activate)
End Sub

Public Sub PlaySound(FileName As String, Channel As Long, LoopSound As Boolean)
    WAVMIX_SetFile FileName, Channel
    WAVMIX_PlayChannel Channel, IIf(LoopSound = True, 1, 0)
End Sub

Sub StopSound(iChannel As Long)
    PlaySound vbNullString, iChannel, False
End Sub
Public Sub WAVMIX_PlayChannel(ChannelNum As Long, LoopWave As Long)
    'Play a specified channel, and indicate whether the sound
    'should be looped.
    
    Dim Params As MIXPLAYPARAMS
    Dim Temp As Long

    If WAVMIX_Quiet Then: Exit Sub
    If ChannelNum > UBound(lpWaveMix) Then: Exit Sub
    If hWaveMix = 0 Then: Exit Sub

    Params.wSize = Len(Params)
    Params.hMixSessionLo = LoWord(hWaveMix)
    Params.hMixSessionHi = HiWord(hWaveMix)
    Params.iChannelLo = LoWord(ChannelNum)
    Params.iChannelHi = HiWord(ChannelNum)
    Params.lpMixWaveLo = LoWord(lpWaveMix(ChannelNum))
    Params.lpMixWaveHi = HiWord(lpWaveMix(ChannelNum))
    Params.hWndNotifyLo = 0
    Params.hWndNotifyHi = 0
    Params.dwFlagsHi = 5
    Params.dwFlagsLo = 0
    Params.wLoops = LoopWave
    
    Temp = WaveMixPlay(Params)
End Sub


